from fastapi import FastAPI, WebSocket
from fastapi.middleware.cors import CORSMiddleware
from mcp.server.fastmcp import FastMCP
from rhino_mcp.rhino_tools import RhinoTools, RhinoConnection
from typing import Dict, Any
import logging
import json
import argparse

# configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("WebServer")

# create FastAPI app
app = FastAPI()

# allow cross-domain requests
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# create MCP instance and tools
mcp = FastMCP("RhinoMCP")
rhino_tools = RhinoTools(mcp)

# add strategy
@mcp.prompt()
def rhino_creation_strategy() -> str:
    """Defines the preferred strategy for creating and managing objects in Rhino"""
    return """When working with Rhino through MCP, follow these guidelines:

    Especially when working with geometry, iterate with smaller steps and check the scene state from time to time.
    Act strategically with a long-term plan, think about how to organize the data and scene objects in a way that is easy to maintain and extend, by using layers and metadata (name, description),
    with the get_rhino_objects_with_metadata() function you can filter and select objects based on this metadata. You can access objects, and with the "type" attribute you can check their geometry type and
    access the geometry specific properties (such as corner points etc.) to create more complex scenes with spatial consistency. Start from sparse to detail (e.g. first the building plot, then the wall, then the window etc. - it is crucial to use metadata to be able to do that)

    1. Scene Context Awareness:
       - Always start by checking the scene using get_rhino_scene_info() for basic overview
       - Use the capture_rhino_viewport to get an image from viewport to get a quick overview of the scene
       - Use get_rhino_objects_with_metadata() for detailed object information and filtering
       - The short_id in metadata can be displayed in viewport using capture_rhino_viewport()

    2. Object Creation and Management:
       - When creating objects, ALWAYS call add_rhino_object_metadata() after creation (The add_rhino_object_metadata() function is provided in the code context)   
       - Use meaningful names for objects to help with you with later identification, organize the scenes with layers (but not too many layers)
       - Think about grouping objects (e.g. two planes that form a window)
    
    3. Always check the bbox for each item so that (it's stored as list of points in the metadata under the key "bbox"):
            - Ensure that all objects that should not be clipping are not clipping.
            - Items have the right spatial relationship.

    4. Code Execution:
       - This is Rhino 7 with IronPython 2.7 - no f-strings or modern Python features etc
       - DONT FORGET NO f-strings! No f-strings, No f-strings!
       - Prefer automated solutions over user interaction, unless its requested or it makes sense or you struggle with errors
       - You can use rhino command syntax to ask the user questions e.g. "should i do "A" or "B"" where A,B are clickable options

    5. Best Practices:
       - Keep objects organized in appropriate layers
       - Use meaningful names and descriptions
       - Use viewport captures to verify visual results
    """


# HTTP endpoint
@app.post("/rhino/command")
async def execute_command(command: Dict[str, Any]):
    """execute Rhino command"""
    try:
        result = rhino_tools.execute_command(command)
        return {"status": "success", "data": result}
    except Exception as e:
        logger.error(f"Command execution error: {str(e)}")
        return {"status": "error", "message": str(e)}

@app.get("/rhino/scene")
async def get_scene():
    """get scene info"""
    try:
        scene_info = rhino_tools.get_rhino_scene_info()
        return {"status": "success", "data": scene_info}
    except Exception as e:
        return {"status": "error", "message": str(e)}

@app.get("/rhino/strategy")
async def get_strategy():
    """get Rhino strategy"""
    return {
        "rhino_strategy": rhino_creation_strategy(),
    }

# WebSocket endpoint
@app.websocket("/rhino/ws")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()
    rhino_conn = RhinoConnection(port=9876)
    
    try:
        # connect to Rhino
        rhino_conn.connect()
        logger.info("Connected to Rhino socket server")
        
        # send initial connection success message
        await websocket.send_json({
            "status": "connected",
            "message": "Connected to Rhino socket server"
        })
        
        # main message loop
        while True:
            try:
                # wait for message
                data = await websocket.receive_json()
                logger.info(f"Received command: {data}")
                
                # send command to Rhino
                result = rhino_conn.send_command(data["type"], data.get("params", {}))
                
                # send result
                await websocket.send_json({
                    "status": "success",
                    "data": result
                })
                
            except json.JSONDecodeError:
                await websocket.send_json({
                    "status": "error",
                    "message": "Invalid JSON format"
                })
            except Exception as e:
                logger.error(f"Command execution error: {str(e)}")
                await websocket.send_json({
                    "status": "error",
                    "message": str(e)
                })
                
    except Exception as e:
        logger.error(f"WebSocket error: {str(e)}")
        try:
            await websocket.send_json({
                "status": "error",
                "message": f"Connection error: {str(e)}"
            })
        except:
            pass
            
    finally:
        # clean up connection
        try:
            rhino_conn.disconnect()
            logger.info("Disconnected from Rhino socket server")
            await websocket.close()
        except:
            pass

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--port", type=int, default=8000, help="Web server port")
    parser.add_argument("--host", type=str, default="localhost", help="Web server host")
    args = parser.parse_args()

    import uvicorn
    uvicorn.run(app, host=args.host, port=args.port) 


"""Rhino integration through the Model Context Protocol."""
from mcp.server.fastmcp import FastMCP, Context, Image
import logging
from dataclasses import dataclass
from contextlib import asynccontextmanager
from typing import AsyncIterator, Dict, Any, List, Optional
from pathlib import Path
from rhino_mcp.rhino_tools import RhinoTools, get_rhino_connection
from typing import Any
from mcp.server.fastmcp import FastMCP
from starlette.applications import Starlette
from mcp.server.sse import SseServerTransport
from starlette.requests import Request
from starlette.routing import Mount, Route
from mcp.server import Server
import uvicorn

# Try to load environment variables from .env file
try:
    from dotenv import load_dotenv
    env_path = Path(__file__).parent.parent / '.env'
    if env_path.exists():
        load_dotenv(dotenv_path=env_path)
        logging.info(f"Loaded environment variables from {env_path}")
except ImportError:
    logging.warning("python-dotenv not installed. Install it to use .env files: pip install python-dotenv")



# Configure logging
logging.basicConfig(level=logging.INFO, 
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger("RhinoMCPServer")

@asynccontextmanager
async def server_lifespan(server: FastMCP) -> AsyncIterator[Dict[str, Any]]:
    """Manage server startup and shutdown lifecycle"""
    rhino_conn = None
    gh_conn = None
    
    try:
        logger.info("RhinoMCP server starting up")
        
        # Try to connect to Rhino script
        try:
            rhino_conn = get_rhino_connection()
            rhino_conn.connect()
            logger.info("Successfully connected to Rhino script")
        except Exception as e:
            logger.warning("Could not connect to Rhino script: {0}".format(str(e)))
        
        yield {}
    finally:
        logger.info("RhinoMCP server shut down")
        
        # Clean up connections
        if rhino_conn:
            try:
                rhino_conn.disconnect()
                logger.info("Disconnected from Rhino script")
            except Exception as e:
                logger.warning("Error disconnecting from Rhino: {0}".format(str(e)))

# Create the MCP server with lifespan support
app = FastMCP(
    "RhinoMCP",
    description="Rhino integration through the Model Context Protocol",
    lifespan=server_lifespan
)

# Initialize tool collections
rhino_tools = RhinoTools(app)

@app.prompt()
def rhino_creation_strategy() -> str:
    """Defines the preferred strategy for creating and managing objects in Rhino"""
    return """When working with Rhino through MCP, follow these guidelines:

    Especially when working with geometry, iterate with smaller steps and check the scene state from time to time.
    Act strategically with a long-term plan, think about how to organize the data and scene objects in a way that is easy to maintain and extend, by using layers and metadata (name, description),
    with the get_rhino_objects_with_metadata() function you can filter and select objects based on this metadata. You can access objects, and with the "type" attribute you can check their geometry type and
    access the geometry specific properties (such as corner points etc.) to create more complex scenes with spatial consistency. Start from sparse to detail (e.g. first the building plot, then the wall, then the window etc. - it is crucial to use metadata to be able to do that)

    1. Scene Context Awareness:
       - Always start by checking the scene using get_rhino_scene_info() for basic overview
       - Use the capture_rhino_viewport to get an image from viewport to get a quick overview of the scene
       - Use get_rhino_objects_with_metadata() for detailed object information and filtering
       - The short_id in metadata can be displayed in viewport using capture_rhino_viewport()

    2. Object Creation and Management:
       - When creating objects, ALWAYS call add_rhino_object_metadata() after creation (The add_rhino_object_metadata() function is provided in the code context)   
       - Use meaningful names for objects to help with you with later identification, organize the scenes with layers (but not too many layers)
       - Think about grouping objects (e.g. two planes that form a window)
    
    3. Always check the bbox for each item so that (it's stored as list of points in the metadata under the key "bbox"):
            - Ensure that all objects that should not be clipping are not clipping.
            - Items have the right spatial relationship.

    4. Code Execution:
       - This is Rhino 7 with IronPython 2.7 - no f-strings or modern Python features etc
       - Please use the Irhono python 2.7 library to access rhino functions
       - DONT FORGET NO f-strings! No f-strings, No f-strings!
       - Prefer automated solutions over user interaction, unless its requested or it makes sense or you struggle with errors
       - You can use rhino command syntax to ask the user questions e.g. "should i do "A" or "B"" where A,B are clickable options

    5. Best Practices:
       - Keep objects organized in appropriate layers
       - Use meaningful names and descriptions
       - Use viewport captures to verify visual results
    """

def create_starlette_app(mcp_server: Server, *, debug: bool = False) -> Starlette:
    """Create a Starlette application that can serve the provided mcp server with SSE."""
    sse = SseServerTransport("/messages/")

    async def handle_sse(request: Request) -> None:
        async with sse.connect_sse(
                request.scope,
                request.receive,
                request._send,
        ) as (read_stream, write_stream):
            await mcp_server.run(
                read_stream,
                write_stream,
                mcp_server.create_initialization_options(),
            )

    return Starlette(
        debug=debug,
        routes=[
            Route("/sse", endpoint=handle_sse),
            Mount("/messages/", app=sse.handle_post_message),
        ],
    )

# Replace the main function
def main():
    """Run the MCP server with SSE transport"""
    mcp_server = app._mcp_server

    # Define default settings
    HOST = "127.0.0.1"  
    PORT = 8080      

    # Create and run the Starlette app
    starlette_app = create_starlette_app(mcp_server, debug=True)
    logger.info(f"Starting RhinoMCP server on {HOST}:{PORT}")
    uvicorn.run(starlette_app, host=HOST, port=PORT)

if __name__ == "__main__":
    main()


"""
This module provides a mapping of RhinoScriptSyntax functions to their categories.
Generated from the RhinoScriptSyntax documentation.
"""

def get_function_category(function_name):
    """
    Returns the category a RhinoScriptSyntax function belongs to.
    
    Args:
        function_name (str): The name of a RhinoScriptSyntax function
        
    Returns:
        str: The category name or None if function not found
    """
    if function_name in FUNCTION_CATEGORY_MAP:
        return FUNCTION_CATEGORY_MAP[function_name]
    return None

# Dictionary mapping function names to their categories
FUNCTION_CATEGORY_MAP = {
    # application
    "AddAlias": "application",
    "AddSearchPath": "application",
    "AliasCount": "application",
    "AliasMacro": "application",
    "AliasNames": "application",
    "AppearanceColor": "application",
    "AutosaveFile": "application",
    "AutosaveInterval": "application",
    "BuildDate": "application",
    "ClearCommandHistory": "application",
    "Command": "application",
    "CommandHistory": "application",
    "DefaultRenderer": "application",
    "DeleteAlias": "application",
    "DeleteSearchPath": "application",
    "DisplayOleAlerts": "application",
    "EdgeAnalysisColor": "application",
    "EdgeAnalysisMode": "application",
    "EnableAutosave": "application",
    "EnablePlugIn": "application",
    "ExeFolder": "application",
    "ExePlatform": "application",
    "ExeServiceRelease": "application",
    "ExeVersion": "application",
    "Exit": "application",
    "FindFile": "application",
    "GetPlugInObject": "application",
    "InCommand": "application",
    "InstallFolder": "application",
    "IsAlias": "application",
    "IsCommand": "application",
    "IsPlugIn": "application",
    "IsRunningOnWindows": "application",
    "LastCommandName": "application",
    "LastCommandResult": "application",
    "LocaleID": "application",
    "Ortho": "application",
    "Osnap": "application",
    "OsnapDialog": "application",
    "OsnapMode": "application",
    "Planar": "application",
    "PlugInId": "application",
    "PlugIns": "application",
    "ProjectOsnaps": "application",
    "Prompt": "application",
    "ScreenSize": "application",
    "SdkVersion": "application",
    "SearchPathCount": "application",
    "SearchPathList": "application",
    "SendKeystrokes": "application",
    "Snap": "application",
    "StatusBarDistance": "application",
    "StatusBarMessage": "application",
    "StatusBarPoint": "application",
    "StatusBarProgressMeterShow": "application",
    "StatusBarProgressMeterUpdate": "application",
    "StatusBarProgressMeterHide": "application",
    "TemplateFile": "application",
    "TemplateFolder": "application",
    "WindowHandle": "application",
    "WorkingFolder": "application",
    
    # block
    "AddBlock": "block",
    "BlockContainerCount": "block",
    "BlockContainers": "block",
    "BlockCount": "block",
    "BlockDescription": "block",
    "BlockInstanceCount": "block",
    "BlockInstanceInsertPoint": "block",
    "BlockInstanceName": "block",
    "BlockInstances": "block",
    "BlockInstanceXform": "block",
    "BlockNames": "block",
    "BlockObjectCount": "block",
    "BlockObjects": "block",
    "BlockPath": "block",
    "BlockStatus": "block",
    "DeleteBlock": "block",
    "ExplodeBlockInstance": "block",
    "InsertBlock": "block",
    "InsertBlock2": "block",
    "IsBlock": "block",
    "IsBlockEmbedded": "block",
    "IsBlockInstance": "block",
    "IsBlockInUse": "block",
    "IsBlockReference": "block",
    "RenameBlock": "block",

    # curve
    "AddArc": "curve",
    "AddArc3Pt": "curve",
    "AddArcPtTanPt": "curve",
    "AddBlendCurve": "curve",
    "AddCircle": "curve",
    "AddCircle3Pt": "curve",
    "AddCurve": "curve",
    "AddEllipse": "curve",
    "AddEllipse3Pt": "curve",
    "AddFilletCurve": "curve",
    "AddInterpCrvOnSrf": "curve",
    "AddInterpCrvOnSrfUV": "curve",
    "AddInterpCurve": "curve",
    "AddLine": "curve",
    "AddNurbsCurve": "curve",
    "AddPolyline": "curve",
    "AddRectangle": "curve",
    "AddSpiral": "curve",
    "AddSubCrv": "curve",
    "ArcAngle": "curve",
    "ArcCenterPoint": "curve",
    "ArcMidPoint": "curve",
    "ArcRadius": "curve",
    "CircleCenterPoint": "curve",
    "CircleCircumference": "curve",
    "CircleRadius": "curve",
    "CloseCurve": "curve",
    "ClosedCurveOrientation": "curve",
    "ConvertCurveToPolyline": "curve",
    "CurveArcLengthPoint": "curve",
    "CurveArea": "curve",
    "CurveAreaCentroid": "curve",
    "CurveArrows": "curve",
    "CurveBooleanDifference": "curve",
    "CurveBooleanIntersection": "curve",
    "CurveBooleanUnion": "curve",
    "CurveBrepIntersect": "curve",
    "CurveClosestObject": "curve",
    "CurveClosestPoint": "curve",
    "CurveContourPoints": "curve",
    "CurveCurvature": "curve",
    "CurveCurveIntersection": "curve",
    "CurveDegree": "curve",
    "CurveDeviation": "curve",
    "CurveDim": "curve",
    "CurveDirectionsMatch": "curve",
    "CurveDiscontinuity": "curve",
    "CurveDomain": "curve",
    "CurveEditPoints": "curve",
    "CurveEndPoint": "curve",
    "CurveFilletPoints": "curve",
    "CurveFrame": "curve",
    "CurveKnotCount": "curve",
    "CurveKnots": "curve",
    "CurveLength": "curve",
    "CurveMidPoint": "curve",
    "CurveNormal": "curve",
    "CurveNormalizedParameter": "curve",
    "CurveParameter": "curve",
    "CurvePerpFrame": "curve",
    "CurvePlane": "curve",
    "CurvePointCount": "curve",
    "CurvePoints": "curve",
    "CurveRadius": "curve",
    "CurveSeam": "curve",
    "CurveStartPoint": "curve",
    "CurveSurfaceIntersection": "curve",
    "CurveTangent": "curve",
    "CurveWeights": "curve",
    "DivideCurve": "curve",
    "DivideCurveEquidistant": "curve",
    "DivideCurveLength": "curve",
    "EllipseCenterPoint": "curve",
    "EllipseQuadPoints": "curve",
    "EvaluateCurve": "curve",
    "ExplodeCurves": "curve",
    "ExtendCurve": "curve",
    "ExtendCurveLength": "curve",
    "ExtendCurvePoint": "curve",
    "FairCurve": "curve",
    "FitCurve": "curve",
    "InsertCurveKnot": "curve",
    "IsArc": "curve",
    "IsCircle": "curve",
    "IsCurve": "curve",
    "IsCurveClosable": "curve",
    "IsCurveClosed": "curve",
    "IsCurveInPlane": "curve",
    "IsCurveLinear": "curve",
    "IsCurvePeriodic": "curve",
    "IsCurvePlanar": "curve",
    "IsCurveRational": "curve",
    "IsEllipse": "curve",
    "IsLine": "curve",
    "IsPointOnCurve": "curve",
    "IsPolyCurve": "curve",
    "IsPolyline": "curve",
    "JoinCurves": "curve",
    "LineFitFromPoints": "curve",
    "MakeCurveNonPeriodic": "curve",
    "MeanCurve": "curve",
    "MeshPolyline": "curve",
    "OffsetCurve": "curve",
    "OffsetCurveOnSurface": "curve",
    "PlanarClosedCurveContainment": "curve",
    "PlanarCurveCollision": "curve",
    "PointInPlanarClosedCurve": "curve",
    "PolyCurveCount": "curve",
    "PolylineVertices": "curve",
    "ProjectCurveToMesh": "curve",
    "ProjectCurveToSurface": "curve",
    "RebuildCurve": "curve",
    "RemoveCurveKnot": "curve",
    "ReverseCurve": "curve",
    "SimplifyCurve": "curve",
    "SplitCurve": "curve",
    "TrimCurve": "curve",
    "ChangeCurveDegree": "curve",
    "AddTweenCurves": "curve",
    
    # dimension
    "AddAlignedDimension": "dimension",
    "AddDimStyle": "dimension",
    "AddLeader": "dimension",
    "AddLinearDimension": "dimension",
    "CurrentDimStyle": "dimension",
    "DeleteDimStyle": "dimension",
    "DimensionStyle": "dimension",
    "DimensionText": "dimension",
    "DimensionUserText": "dimension",
    "DimensionValue": "dimension",
    "DimStyleAnglePrecision": "dimension",
    "DimStyleArrowSize": "dimension",
    "DimStyleCount": "dimension",
    "DimStyleExtension": "dimension",
    "DimStyleFont": "dimension",
    "DimStyleLeaderArrowSize": "dimension",
    "DimStyleLengthFactor": "dimension",
    "DimStyleLinearPrecision": "dimension",
    "DimStyleNames": "dimension",
    "DimStyleNumberFormat": "dimension",
    "DimStyleOffset": "dimension",
    "DimStylePrefix": "dimension",
    "DimStyleScale": "dimension",
    "DimStyleSuffix": "dimension",
    "DimStyleTextAlignment": "dimension",
    "DimStyleTextGap": "dimension",
    "DimStyleTextHeight": "dimension",
    "IsAlignedDimension": "dimension",
    "IsAngularDimension": "dimension",
    "IsDiameterDimension": "dimension",
    "IsDimension": "dimension",
    "IsDimStyle": "dimension",
    "IsDimStyleReference": "dimension",
    "IsLeader": "dimension",
    "IsLinearDimension": "dimension",
    "IsOrdinateDimension": "dimension",
    "IsRadialDimension": "dimension",
    "LeaderText": "dimension",
    "RenameDimStyle": "dimension",
    
    # document
    "CreatePreviewImage": "document",
    "DocumentModified": "document",
    "DocumentName": "document",
    "DocumentPath": "document",
    "EnableRedraw": "document",
    "ExtractPreviewImage": "document",
    "IsDocumentModified": "document",
    "Notes": "document",
    "ReadFileVersion": "document",
    "Redraw": "document",
    "RenderAntialias": "document",
    "RenderColor": "document",
    "RenderResolution": "document",
    "RenderMeshDensity": "document",
    "RenderMeshMaxAngle": "document",
    "RenderMeshMaxAspectRatio": "document",
    "RenderMeshMaxDistEdgeToSrf": "document",
    "RenderMeshMaxEdgeLength": "document",
    "RenderMeshMinEdgeLength": "document",
    "RenderMeshMinInitialGridQuads": "document",
    "RenderMeshQuality": "document",
    "RenderMeshSettings": "document",
    "RenderSettings": "document",
    "UnitAbsoluteTolerance": "document",
    "UnitAngleTolerance": "document",
    "UnitDistanceDisplayPrecision": "document",
    "UnitRelativeTolerance": "document",
    "UnitScale": "document",
    "UnitSystem": "document",
    "UnitSystemName": "document",
    
    # geometry
    "AddClippingPlane": "geometry",
    "AddPictureFrame": "geometry",
    "AddPoint": "geometry",
    "AddPointCloud": "geometry",
    "AddPoints": "geometry",
    "AddText": "geometry",
    "AddTextDot": "geometry",
    "Area": "geometry",
    "BoundingBox": "geometry",
    "CompareGeometry": "geometry",
    "ExplodeText": "geometry",
    "IsClippingPlane": "geometry",
    "IsPoint": "geometry",
    "IsPointCloud": "geometry",
    "IsText": "geometry",
    "IsTextDot": "geometry",
    "PointCloudCount": "geometry",
    "PointCloudHasHiddenPoints": "geometry",
    "PointCloudHasPointColors": "geometry",
    "PointCloudHidePoints": "geometry",
    "PointCloudPointColors": "geometry",
    "PointCloudPoints": "geometry",
    "PointCloudKNeighbors": "geometry",
    "PointCloudClosestPoints": "geometry",
    "PointCoordinates": "geometry",
    "TextDotFont": "geometry",
    "TextDotHeight": "geometry",
    "TextDotPoint": "geometry",
    "TextDotText": "geometry",
    "TextObjectFont": "geometry",
    "TextObjectHeight": "geometry",
    "TextObjectPlane": "geometry",
    "TextObjectPoint": "geometry",
    "TextObjectStyle": "geometry",
    "TextObjectText": "geometry",
    
    # grips
    "EnableObjectGrips": "grips",
    "GetObjectGrip": "grips",
    "GetObjectGrips": "grips",
    "NextObjectGrip": "grips",
    "ObjectGripCount": "grips",
    "ObjectGripLocation": "grips",
    "ObjectGripLocations": "grips",
    "ObjectGripsOn": "grips",
    "ObjectGripsSelected": "grips",
    "PrevObjectGrip": "grips",
    "SelectedObjectGrips": "grips",
    "SelectObjectGrip": "grips",
    "SelectObjectGrips": "grips",
    "UnselectObjectGrip": "grips",
    "UnselectObjectGrips": "grips",
    
    # group
    "AddGroup": "group",
    "AddObjectsToGroup": "group",
    "AddObjectToGroup": "group",
    "DeleteGroup": "group",
    "GroupCount": "group",
    "GroupNames": "group",
    "HideGroup": "group",
    "IsGroup": "group",
    "IsGroupEmpty": "group",
    "LockGroup": "group",
    "RemoveObjectFromAllGroups": "group",
    "RemoveObjectFromGroup": "group",
    "RemoveObjectsFromGroup": "group",
    "RenameGroup": "group",
    "ShowGroup": "group",
    "UnlockGroup": "group",
    "ObjectTopGroup": "group",
    
    # hatch
    "AddHatch": "hatch",
    "AddHatches": "hatch",
    "AddHatchPatterns": "hatch",
    "CurrentHatchPattern": "hatch",
    "ExplodeHatch": "hatch",
    "HatchPattern": "hatch",
    "HatchPatternCount": "hatch",
    "HatchPatternDescription": "hatch",
    "HatchPatternFillType": "hatch",
    "HatchPatternNames": "hatch",
    "HatchRotation": "hatch",
    "HatchScale": "hatch",
    "IsHatch": "hatch",
    "IsHatchPattern": "hatch",
    "IsHatchPatternCurrent": "hatch",
    "IsHatchPatternReference": "hatch",
    
    # layer
    "AddLayer": "layer",
    "CurrentLayer": "layer",
    "DeleteLayer": "layer",
    "ExpandLayer": "layer",
    "IsLayer": "layer",
    "IsLayerChangeable": "layer",
    "IsLayerChildOf": "layer",
    "IsLayerCurrent": "layer",
    "IsLayerEmpty": "layer",
    "IsLayerExpanded": "layer",
    "IsLayerLocked": "layer",
    "IsLayerOn": "layer",
    "IsLayerSelectable": "layer",
    "IsLayerParentOf": "layer",
    "IsLayerReference": "layer",
    "IsLayerVisible": "layer",
    "LayerChildCount": "layer",
    "LayerChildren": "layer",
    "LayerColor": "layer",
    "LayerCount": "layer",
    "LayerIds": "layer",
    "LayerLinetype": "layer",
    "LayerLocked": "layer",
    "LayerMaterialIndex": "layer",
    "LayerId": "layer",
    "LayerName": "layer",
    "LayerNames": "layer",
    "LayerOrder": "layer",
    "LayerPrintColor": "layer",
    "LayerPrintWidth": "layer",
    "LayerVisible": "layer",
    "ParentLayer": "layer",
    "PurgeLayer": "layer",
    "RenameLayer": "layer",
    
    # light
    "AddDirectionalLight": "light",
    "AddLinearLight": "light",
    "AddPointLight": "light",
    "AddRectangularLight": "light",
    "AddSpotLight": "light",
    "EnableLight": "light",
    "IsDirectionalLight": "light",
    "IsLight": "light",
    "IsLightEnabled": "light",
    "IsLightReference": "light",
    "IsLinearLight": "light",
    "IsPointLight": "light",
    "IsRectangularLight": "light",
    "IsSpotLight": "light",
    "LightColor": "light",
    "LightCount": "light",
    "LightDirection": "light",
    "LightLocation": "light",
    "LightName": "light",
    "LightObjects": "light",
    "RectangularLightPlane": "light",
    "SpotLightHardness": "light",
    "SpotLightRadius": "light",
    "SpotLightShadowIntensity": "light",
    
    # line
    "LineClosestPoint": "line",
    "LineCylinderIntersection": "line",
    "LineIsFartherThan": "line",
    "LineLineIntersection": "line",
    "LineMaxDistanceTo": "line",
    "LineMinDistanceTo": "line",
    "LinePlane": "line",
    "LinePlaneIntersection": "line",
    "LineSphereIntersection": "line",
    "LineTransform": "line",
    
    # linetype
    "IsLinetype": "linetype",
    "IsLinetypeReference": "linetype",
    "LinetypeCount": "linetype",
    "LinetypeNames": "linetype",
    
    # material
    "AddMaterialToLayer": "material",
    "AddMaterialToObject": "material",
    "CopyMaterial": "material",
    "IsMaterialDefault": "material",
    "IsMaterialReference": "material",
    "MatchMaterial": "material",
    "MaterialBump": "material",
    "MaterialColor": "material",
    "MaterialEnvironmentMap": "material",
    "MaterialName": "material",
    "MaterialReflectiveColor": "material",
    "MaterialShine": "material",
    "MaterialTexture": "material",
    "MaterialTransparency": "material",
    "MaterialTransparencyMap": "material",
    "ResetMaterial": "material",
    
    # mesh
    "AddMesh": "mesh",
    "AddPlanarMesh": "mesh",
    "CurveMeshIntersection": "mesh",
    "DisjointMeshCount": "mesh",
    "DuplicateMeshBorder": "mesh",
    "ExplodeMeshes": "mesh",
    "IsMesh": "mesh",
    "IsMeshClosed": "mesh",
    "IsMeshManifold": "mesh",
    "IsPointOnMesh": "mesh",
    "JoinMeshes": "mesh",
    "MeshArea": "mesh",
    "MeshAreaCentroid": "mesh",
    "MeshBooleanDifference": "mesh",
    "MeshBooleanIntersection": "mesh",
    "MeshBooleanSplit": "mesh",
    "MeshBooleanUnion": "mesh",
    "MeshClosestPoint": "mesh",
    "MeshFaceCenters": "mesh",
    "MeshFaceCount": "mesh",
    "MeshFaceNormals": "mesh",
    "MeshFaces": "mesh",
    "MeshFaceVertices": "mesh",
    "MeshHasFaceNormals": "mesh",
    "MeshHasTextureCoordinates": "mesh",
    "MeshHasVertexColors": "mesh",
    "MeshHasVertexNormals": "mesh",
    "MeshMeshIntersection": "mesh",
    "MeshNakedEdgePoints": "mesh",
    "MeshOffset": "mesh",
    "MeshOutline": "mesh",
    "MeshQuadCount": "mesh",
    "MeshQuadsToTriangles": "mesh",
    "MeshToNurb": "mesh",
    "MeshTriangleCount": "mesh",
    "MeshVertexColors": "mesh",
    "MeshVertexCount": "mesh",
    "MeshVertexFaces": "mesh",
    "MeshVertexNormals": "mesh",
    "MeshVertices": "mesh",
    "MeshVolume": "mesh",
    "MeshVolumeCentroid": "mesh",
    "PullCurveToMesh": "mesh",
    "SplitDisjointMesh": "mesh",
    "UnifyMeshNormals": "mesh",
    
    # object
    "CopyObject": "object",
    "CopyObjects": "object",
    "DeleteObject": "object",
    "DeleteObjects": "object",
    "FlashObject": "object",
    "HideObject": "object",
    "HideObjects": "object",
    "IsLayoutObject": "object",
    "IsObject": "object",
    "IsObjectHidden": "object",
    "IsObjectInBox": "object",
    "IsObjectInGroup": "object",
    "IsObjectLocked": "object",
    "IsObjectNormal": "object",
    "IsObjectReference": "object",
    "IsObjectSelectable": "object",
    "IsObjectSelected": "object",
    "IsObjectSolid": "object",
    "IsObjectValid": "object",
    "IsVisibleInView": "object",
    "LockObject": "object",
    "LockObjects": "object",
    "MatchObjectAttributes": "object",
    "MirrorObject": "object",
    "MirrorObjects": "object",
    "MoveObject": "object",
    "MoveObjects": "object",
    "ObjectColor": "object",
    "ObjectColorSource": "object",
    "ObjectDescription": "object",
    "ObjectGroups": "object",
    "ObjectLayer": "object",
    "ObjectLayout": "object",
    "ObjectLinetype": "object",
    "ObjectLinetypeSource": "object",
    "ObjectMaterialIndex": "object",
    "ObjectMaterialSource": "object",
    "ObjectName": "object",
    "ObjectPrintColor": "object",
    "ObjectPrintColorSource": "object",
    "ObjectPrintWidth": "object",
    "ObjectPrintWidthSource": "object",
    "ObjectType": "object",
    "OrientObject": "object",
    "RotateObject": "object",
    "RotateObjects": "object",
    "ScaleObject": "object",
    "ScaleObjects": "object",
    "SelectObject": "object",
    "SelectObjects": "object",
    "ShearObject": "object",
    "ShearObjects": "object",
    "ShowObject": "object",
    "ShowObjects": "object",
    "TransformObject": "object",
    "TransformObjects": "object",
    "UnlockObject": "object",
    "UnlockObjects": "object",
    "UnselectObject": "object",
    "UnselectObjects": "object",
    
    # plane
    "DistanceToPlane": "plane",
    "EvaluatePlane": "plane",
    "IntersectPlanes": "plane",
    "MovePlane": "plane",
    "PlaneClosestPoint": "plane",
    "PlaneCurveIntersection": "plane",
    "PlaneEquation": "plane",
    "PlaneFitFromPoints": "plane",
    "PlaneFromFrame": "plane",
    "PlaneFromNormal": "plane",
    "PlaneFromPoints": "plane",
    "PlanePlaneIntersection": "plane",
    "PlaneSphereIntersection": "plane",
    "PlaneTransform": "plane",
    "RotatePlane": "plane",
    "WorldXYPlane": "plane",
    "WorldYZPlane": "plane",
    "WorldZXPlane": "plane",
    
    # pointvector
    "IsVectorParallelTo": "pointvector",
    "IsVectorPerpendicularTo": "pointvector",
    "IsVectorTiny": "pointvector",
    "IsVectorZero": "pointvector",
    "PointAdd": "pointvector",
    "PointArrayClosestPoint": "pointvector",
    "PointArrayTransform": "pointvector",
    "PointClosestObject": "pointvector",
    "PointCompare": "pointvector",
    "PointDivide": "pointvector",
    "PointsAreCoplanar": "pointvector",
    "PointScale": "pointvector",
    "PointSubtract": "pointvector",
    "PointTransform": "pointvector",
    "ProjectPointToMesh": "pointvector",
    "ProjectPointToSurface": "pointvector",
    "PullPoints": "pointvector",
    "VectorAdd": "pointvector",
    "VectorAngle": "pointvector",
    "VectorCompare": "pointvector",
    "VectorCreate": "pointvector",
    "VectorCrossProduct": "pointvector",
    "VectorDivide": "pointvector",
    "VectorDotProduct": "pointvector",
    "VectorLength": "pointvector",
    "VectorMultiply": "pointvector",
    "VectorReverse": "pointvector",
    "VectorRotate": "pointvector",
    "VectorScale": "pointvector",
    "VectorSubtract": "pointvector",
    "VectorTransform": "pointvector",
    "VectorUnitize": "pointvector",
    "PointArrayBoundingBox": "pointvector",
    
    # selection
    "AllObjects": "selection",
    "FirstObject": "selection",
    "GetCurveObject": "selection",
    "GetObject": "selection",
    "GetObjectEx": "selection",
    "GetObjects": "selection",
    "GetObjectsEx": "selection",
    "GetPointCoordinates": "selection",
    "GetSurfaceObject": "selection",
    "LockedObjects": "selection",
    "HiddenObjects": "selection",
    "InvertSelectedObjects": "selection",
    "LastCreatedObjects": "selection",
    "LastObject": "selection",
    "NextObject": "selection",
    "NormalObjects": "selection",
    "ObjectsByColor": "selection",
    "ObjectsByGroup": "selection",
    "ObjectsByLayer": "selection",
    "ObjectsByName": "selection",
    "ObjectsByType": "selection",
    "SelectedObjects": "selection",
    "UnselectAllObjects": "selection",
    "VisibleObjects": "selection",
    "WindowPick": "selection",
    
    # surface
    "AddBox": "surface",
    "AddCone": "surface",
    "AddCutPlane": "surface",
    "AddCylinder": "surface",
    "AddEdgeSrf": "surface",
    "AddNetworkSrf": "surface",
    "AddNurbsSurface": "surface",
    "AddPatch": "surface",
    "AddPipe": "surface",
    "AddPlanarSrf": "surface",
    "AddPlaneSurface": "surface",
    "AddLoftSrf": "surface",
    "AddRevSrf": "surface",
    "AddSphere": "surface",
    "AddSrfContourCrvs": "surface",
    "AddSrfControlPtGrid": "surface",
    "AddSrfPt": "surface",
    "AddSrfPtGrid": "surface",
    "AddSweep1": "surface",
    "AddSweep2": "surface",
    "AddRailRevSrf": "surface",
    "AddTorus": "surface",
    "BooleanDifference": "surface",
    "BooleanIntersection": "surface",
    "BooleanUnion": "surface",
    "BrepClosestPoint": "surface",
    "CapPlanarHoles": "surface",
    "DuplicateEdgeCurves": "surface",
    "DuplicateSurfaceBorder": "surface",
    "EvaluateSurface": "surface",
    "ExtendSurface": "surface",
    "ExplodePolysurfaces": "surface",
    "ExtractIsoCurve": "surface",
    "ExtractSurface": "surface",
    "ExtrudeCurve": "surface",
    "ExtrudeCurvePoint": "surface",
    "ExtrudeCurveStraight": "surface",
    "ExtrudeSurface": "surface",
    "FilletSurfaces": "surface",
    "FlipSurface": "surface",
    "IntersectBreps": "surface",
    "IntersectSpheres": "surface",
    "IsBrep": "surface",
    "IsCone": "surface",
    "IsCylinder": "surface",
    "IsPlaneSurface": "surface",
    "IsPointInSurface": "surface",
    "IsPointOnSurface": "surface",
    "IsPolysurface": "surface",
    "IsPolysurfaceClosed": "surface",
    "IsSphere": "surface",
    "IsSurface": "surface",
    "IsSurfaceClosed": "surface",
    "IsSurfacePeriodic": "surface",
    "IsSurfacePlanar": "surface",
    "IsSurfaceRational": "surface",
    "IsSurfaceSingular": "surface",
    "IsSurfaceTrimmed": "surface",
    "IsTorus": "surface",
    "SurfaceSphere": "surface",
    "JoinSurfaces": "surface",
    "MakeSurfacePeriodic": "surface",
    "OffsetSurface": "surface",
    "PullCurve": "surface",
    "RebuildSurface": "surface",
    "RemoveSurfaceKnot": "surface",
    "ReverseSurface": "surface",
    "ShootRay": "surface",
    "ShortPath": "surface",
    "ShrinkTrimmedSurface": "surface",
    "SplitBrep": "surface",
    "SurfaceArea": "surface",
    "SurfaceAreaCentroid": "surface",
    "SurfaceAreaMoments": "surface",
    "SurfaceClosestPoint": "surface",
    "SurfaceCone": "surface",
    "SurfaceCurvature": "surface",
    "SurfaceCylinder": "surface",
    "SurfaceDegree": "surface",
    "SurfaceDomain": "surface",
    "SurfaceEditPoints": "surface",
    "SurfaceEvaluate": "surface",
    "SurfaceFrame": "surface",
    "SurfaceIsocurveDensity": "surface",
    "SurfaceKnotCount": "surface",
    "SurfaceKnots": "surface",
    "SurfaceNormal": "surface",
    "SurfaceNormalizedParameter": "surface",
    "SurfaceParameter": "surface",
    "SurfacePointCount": "surface",
    "SurfacePoints": "surface",
    "SurfaceTorus": "surface",
    "SurfaceVolume": "surface",
    "SurfaceVolumeCentroid": "surface",
    "SurfaceVolumeMoments": "surface",
    "SurfaceWeights": "surface",
    "TrimBrep": "surface",
    "TrimSurface": "surface",
    "UnrollSurface": "surface",
    "ChangeSurfaceDegree": "surface",
    
    # toolbar
    "CloseToolbarCollection": "toolbar",
    "HideToolbar": "toolbar",
    "IsToolbar": "toolbar",
    "IsToolbarCollection": "toolbar",
    "IsToolbarDocked": "toolbar",
    "IsToolbarVisible": "toolbar",
    "OpenToolbarCollection": "toolbar",
    "SaveToolbarCollection": "toolbar",
    "SaveToolbarCollectionAs": "toolbar",
    "ShowToolbar": "toolbar",
    "ToolbarCollectionCount": "toolbar",
    "ToolbarCollectionNames": "toolbar",
    "ToolbarCollectionPath": "toolbar",
    "ToolbarCount": "toolbar",
    "ToolbarNames": "toolbar",
    
    # transformation
    "IsXformIdentity": "transformation",
    "IsXformSimilarity": "transformation",
    "IsXformZero": "transformation",
    "XformChangeBasis": "transformation",
    "XformChangeBasis2": "transformation",
    "XformCompare": "transformation",
    "XformCPlaneToWorld": "transformation",
    "XformDeterminant": "transformation",
    "XformDiagonal": "transformation",
    "XformIdentity": "transformation",
    "XformInverse": "transformation",
    "XformMirror": "transformation",
    "XformMultiply": "transformation",
    "XformPlanarProjection": "transformation",
    "XformRotation1": "transformation",
    "XformRotation2": "transformation",
    "XformRotation3": "transformation",
    "XformRotation4": "transformation",
    "XformScale": "transformation",
    "XformScreenToWorld": "transformation",
    "XformShear": "transformation",
    "XformTranslation": "transformation",
    "XformWorldToCPlane": "transformation",
    "XformWorldToScreen": "transformation",
    "XformZero": "transformation",
    
    # userdata
    "DeleteDocumentData": "userdata",
    "DocumentDataCount": "userdata",
    "DocumentUserTextCount": "userdata",
    "GetDocumentData": "userdata",
    "GetDocumentUserText": "userdata",
    "GetUserText": "userdata",
    "IsDocumentData": "userdata",
    "IsDocumentUserText": "userdata",
    "IsUserText": "userdata",
    "SetDocumentData": "userdata",
    "SetDocumentUserText": "userdata",
    "SetUserText": "userdata",
    
    # userinterface
    "BrowseForFolder": "userinterface",
    "CheckListBox": "userinterface",
    "ComboListBox": "userinterface",
    "EditBox": "userinterface",
    "GetAngle": "userinterface",
    "GetBoolean": "userinterface",
    "GetBox": "userinterface",
    "GetColor": "userinterface",
    "GetCursorPos": "userinterface",
    "GetDistance": "userinterface",
    "GetEdgeCurves": "userinterface",
    "GetInteger": "userinterface",
    "GetLayer": "userinterface",
    "GetLayers": "userinterface",
    "GetLine": "userinterface",
    "GetLinetype": "userinterface",
    "GetMeshFaces": "userinterface",
    "GetMeshVertices": "userinterface",
    "GetPoint": "userinterface",
    "GetPointOnCurve": "userinterface",
    "GetPointOnMesh": "userinterface",
    "GetPointOnSurface": "userinterface",
    "GetPoints": "userinterface",
    "GetPolyline": "userinterface",
    "GetReal": "userinterface",
    "GetRectangle": "userinterface",
    "GetString": "userinterface",
    "ListBox": "userinterface",
    "MessageBox": "userinterface",
    "PropertyListBox": "userinterface",
    "MultiListBox": "userinterface",
    "OpenFileName": "userinterface",
    "OpenFileNames": "userinterface",
    "PopupMenu": "userinterface",
    "RealBox": "userinterface",
    "SaveFileName": "userinterface",
    "StringBox": "userinterface",
    "TextOut": "userinterface",
    
    # utility
    "ContextIsRhino": "utility",
    "ContextIsGrasshopper": "utility",
    "Angle": "utility",
    "Angle2": "utility",
    "ClipboardText": "utility",
    "ColorAdjustLuma": "utility",
    "ColorBlueValue": "utility",
    "ColorGreenValue": "utility",
    "ColorHLSToRGB": "utility",
    "ColorRedValue": "utility",
    "ColorRGBToHLS": "utility",
    "CullDuplicateNumbers": "utility",
    "CullDuplicatePoints": "utility",
    "Distance": "utility",
    "GetSettings": "utility",
    "Polar": "utility",
    "SimplifyArray": "utility",
    "Sleep": "utility",
    "SortPointList": "utility",
    "SortPoints": "utility",
    "Str2Pt": "utility",
    "CreatePoint": "utility",
    "CreateVector": "utility",
    "CreatePlane": "utility",
    "CreateXform": "utility",
    "CreateColor": "utility",
    "CreateInterval": "utility",
    
    # view
    "AddDetail": "view",
    "AddLayout": "view",
    "AddNamedCPlane": "view",
    "AddNamedView": "view",
    "CurrentDetail": "view",
    "CurrentView": "view",
    "DeleteNamedCPlane": "view",
    "DeleteNamedView": "view",
    "DetailLock": "view",
    "DetailScale": "view",
    "IsDetail": "view",
    "IsLayout": "view",
    "IsView": "view",
    "IsViewCurrent": "view",
    "IsViewMaximized": "view",
    "IsViewPerspective": "view",
    "IsViewTitleVisible": "view",
    "IsWallpaper": "view",
    "MaximizeRestoreView": "view",
    "NamedCPlane": "view",
    "NamedCPlanes": "view",
    "NamedViews": "view",
    "RenameView": "view",
    "RestoreNamedCPlane": "view",
    "RestoreNamedView": "view",
    "RotateCamera": "view",
    "RotateView": "view",
    "ShowGrid": "view",
    "ShowGridAxes": "view",
    "ShowViewTitle": "view",
    "ShowWorldAxes": "view",
    "TiltView": "view",
    "ViewCamera": "view",
    "ViewCameraLens": "view",
    "ViewCameraPlane": "view",
    "ViewCameraTarget": "view",
    "ViewCameraUp": "view",
    "ViewCPlane": "view",
    "ViewDisplayMode": "view",
    "ViewDisplayModeId": "view",
    "ViewDisplayModeName": "view",
    "ViewDisplayModes": "view",
    "ViewNames": "view",
    "ViewNearCorners": "view",
    "ViewProjection": "view",
    "ViewRadius": "view",
    "ViewSize": "view",
    "ViewSpeedTest": "view",
    "ViewTarget": "view",
    "ViewTitle": "view",
    "Wallpaper": "view",
    "WallpaperGrayScale": "view",
    "WallpaperHidden": "view",
    "ZoomBoundingBox": "view",
    "ZoomExtents": "view",
    "ZoomSelected": "view",
}



"""Rhino integration through the Model Context Protocol."""
from mcp.server.fastmcp import FastMCP, Context, Image
import logging
from dataclasses import dataclass
from contextlib import asynccontextmanager
from typing import AsyncIterator, Dict, Any, List, Optional
from pathlib import Path

# Try to load environment variables from .env file
try:
    from dotenv import load_dotenv
    env_path = Path(__file__).parent.parent / '.env'
    if env_path.exists():
        load_dotenv(dotenv_path=env_path)
        logging.info(f"Loaded environment variables from {env_path}")
except ImportError:
    logging.warning("python-dotenv not installed. Install it to use .env files: pip install python-dotenv")

# Import our tool modules
from rhino_mcp.rhino_tools import RhinoTools, get_rhino_connection
from rhino_mcp.grasshopper_tools import GrasshopperTools

# Configure logging
logging.basicConfig(level=logging.INFO, 
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger("RhinoMCPServer")

@asynccontextmanager
async def server_lifespan(server: FastMCP) -> AsyncIterator[Dict[str, Any]]:
    """Manage server startup and shutdown lifecycle"""
    rhino_conn = None
    
    try:
        logger.info("RhinoMCP server starting up")
        
        # Try to connect to Rhino script
        try:
            rhino_conn = get_rhino_connection()
            rhino_conn.connect()
            logger.info("Successfully connected to Rhino")
        except Exception as e:
            logger.warning("Could not connect to Rhino: {0}".format(str(e)))

        yield {}
    finally:
        logger.info("RhinoMCP server shut down")
        
        # Clean up connections
        if rhino_conn:
            try:
                rhino_conn.disconnect()
                logger.info("Disconnected from Rhino")
            except Exception as e:
                logger.warning("Error disconnecting from Rhino: {0}".format(str(e)))

# Create the MCP server with lifespan support
mcp = FastMCP(
    "RhinoMCP",
    instructions="Rhino integration through the Model Context Protocol",
    lifespan=server_lifespan
)

# Initialize tool collections
rhino_tools = RhinoTools(mcp)
grasshopper_tools = GrasshopperTools(mcp)

@mcp.prompt()
def rhino_creation_strategy() -> str:
    """Defines the preferred strategy for creating and managing objects in Rhino"""
    return """When working with Rhino through MCP, follow these guidelines:

    Especially when working with geometry, iterate with smaller steps and check the scene state from time to time.
    Act strategically with a long-term plan, think about how to organize the data and scene objects in a way that is easy to maintain and extend, by using layers and metadata (name, description),
    with the get_rhino_objects_with_metadata() function you can filter and select objects based on this metadata. You can access objects, and with the "type" attribute you can check their geometry type and
    access the geometry specific properties (such as corner points etc.) to create more complex scenes with spatial consistency. Start from sparse to detail (e.g. first the building plot, then the wall, then the window etc. - it is crucial to use metadata to be able to do that)

    1. Scene Context Awareness:
       - Always start by checking the scene using get_rhino_scene_info() for basic overview
       - Use the capture_rhino_viewport to get an image from viewport to get a quick overview of the scene
       - Use get_rhino_objects_with_metadata() for detailed object information and filtering
       - The short_id in metadata can be displayed in viewport using capture_rhino_viewport()

    2. Object Creation and Management:
       - When creating objects, ALWAYS call add_rhino_object_metadata() after creation (The add_rhino_object_metadata() function is provided in the code context)   
       - Use meaningful names for objects to help with you with later identification, organize the scenes with layers (but not too many layers)
       - Think about grouping objects (e.g. two planes that form a window)
    
    3. Always check the bbox for each item so that (it's stored as list of points in the metadata under the key "bbox"):
        - Ensure that all objects that should not be clipping are not clipping.
        - Items have the right spatial relationship.

    4. Code Execution:
       - This is Rhino 7 with IronPython 2.7 - no f-strings or modern Python features etc
       - rhinoscriptsyntax is already imported as rs
       - scriptcontext is already imported as sc
       - json is already imported as json
       - time is already imported as time
       - datetime is already imported as datetim
       - DONT FORGET NO f-strings! No f-strings, No f-strings!
       - Prefer automated solutions over user interaction, unless its requested or it makes sense or you struggle with errors
       - You can always use the look_up_RhinoScriptSyntax() function to look up the documentation for a RhinoScriptSyntax function directly from the Rhino3D developer website.

    5. Best Practices:
       - Always show the user the code you are executing, for example show the input to the execute_rhino_code() function in { `code`: `...`}   
       - Keep objects organized in appropriate layers
       - Use meaningful names and descriptions
       - Use viewport captures to verify visual results
       - When encouonter errors related to RhinoScriptSyntax, make sure to search the web for the correct syntax from the RhinoScriptSyntax API documentation
    """

@mcp.prompt()
def grasshopper_usage_strategy() -> str:
    """Defines the preferred strategy for working with Grasshopper through MCP"""
    return """When working with Grasshopper through MCP, follow these guidelines:
    Grasshooper is closely itnergrated with rhino, you can access rhino objects by referencing them, you can 
    see grasshopper generted geometry in rhino viewport.

    1. Connection Setup:
       - Always check if the Grasshopper server is available by using is_server_available()

    2. Definition Exploration:
       - Use get_definition_info() to get an overview of components and parameters in the definition

    4. Code Execution Guidelines:
       - Always use IronPython 2.7 compatible code (no f-strings, no walrus operator, etc.)
       - you can create grashopper componetns via code 
       - you can access rhino objects by referencing them

    """

def main():
    """Run the MCP server"""
    mcp.run(transport='stdio')

if __name__ == "__main__":
    main()


"""
Rhino Model Context Protocol (MCP) Integration
----------------------------------------------
A package that allows Claude to interact with Rhino through the Model Context Protocol
"""

__version__ = "0.1.0" 

from .web_server import app as web_app 